package jp.co.asterisk.asreader.a3xd.demoapp.fragment.setting;

import android.os.Bundle;

import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import jp.co.asterisk.asreader.a3xd.demoapp.databinding.FragmentRfidSettingBinding;
import jp.co.asterisk.asreader.a3xd.demoapp.fragment.BaseFragment;
import jp.co.asterisk.asreader.a3xd.demoapp.rfid.SerializableAsDeviceRfidFhLbtParam;
import jp.co.asterisk.asreader.a3xd.sdk.AsReaderSDK;
import jp.co.asterisk.asreader.a3xd.sdk.manager.RFIDManagerCallback;
import jp.co.asterisk.asreader.a3xd.sdk.utility.EpcConverter;
import jp.co.asterisk.asreader.a3xd.demoapp.R;


public class RfidSettingFragment extends BaseFragment implements RFIDManagerCallback {

    private final String TAG = RfidSettingFragment.class.getSimpleName();

    private FragmentRfidSettingBinding binding;

    private int minPower;
    private int maxPower;
    private int powerLevel;
    private boolean isFragmentClose = false;

    private SerializableAsDeviceRfidFhLbtParam param;

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        setBackToHomeActionResId(R.id.action_RfidSettingFragment_to_HomeFragment);

        binding = FragmentRfidSettingBinding.inflate(inflater, container, false);

        AsReaderSDK.getInstance().getDeviceManager().getReaderInfo(0xB0);
        param = new SerializableAsDeviceRfidFhLbtParam();

        binding.displayTagRssiToggleButton.setChecked(AsReaderSDK.getInstance().getRFIDManager().getRSSIEnable());

        binding.rfChannelLayout.setOnClickListener(clickListener);
        binding.outputPowerLayout.setOnClickListener(clickListener);
        binding.onOffTimeLayout.setOnClickListener(clickListener);
        binding.stopConditionsLayout.setOnClickListener(clickListener);
        binding.modulationModeLayout.setOnClickListener(clickListener);
        binding.encodingTypeLayout.setOnClickListener(clickListener);
        binding.sessionLayout.setOnClickListener(clickListener);
        binding.antiCollisionLayout.setOnClickListener(clickListener);
        binding.displayTagRssiToggleButton.setOnClickListener(clickListener);

        return binding.getRoot();
    }

    @Override
    public void onResume() {
        super.onResume();
        AsReaderSDK.getInstance().getRFIDManager().setCallback(this);
        AsReaderSDK.getInstance().getDeviceManager().getReaderInfo(0xB0);

        binding.outputPowerTextView.setText("");
        binding.onOffTimeTextView.setText("");
        binding.antiCollisionTextView.setText("");
        binding.rfChannelTextView.setText("");

        int maxTag = AsReaderSDK.getInstance().getRFIDManager().getMaxTagValue();
        int maxTime = AsReaderSDK.getInstance().getRFIDManager().getMaxTimeValue();
        int repeatCycle = AsReaderSDK.getInstance().getRFIDManager().getRepeatCycleValue();

        String stopCondition = maxTag + "/" + maxTime + "/" + repeatCycle;
        binding.stopConditionsTextView.setText(stopCondition);

        int encodingType = getPreferencesHelper().getEncodingType();
        String sEncodingType = EpcConverter.toTypeString(encodingType);
        binding.encodingTypeTextView.setText(sEncodingType);

        setToolbar(true, getString(R.string.toolbar_title_setting), getString(R.string.button_name_update));
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        isFragmentClose = true;
        binding = null;
    }

    @Override
    public void onClickRightButton(View view) {
        super.onClickRightButton(view);
        AsReaderSDK.getInstance().getRFIDManager().updateRegistry();
    }

    private View.OnClickListener clickListener = (View view) -> {
        if (view == binding.rfChannelLayout) {
            navigate(this, R.id.action_RfidSettingFragment_to_RfChannelFragment);

        } else if (view == binding.outputPowerLayout) {
            Bundle args = new Bundle();
            args.putInt(OutputPowerFragment.ARGS_MIN_POWER, minPower);
            args.putInt(OutputPowerFragment.ARGS_MAX_POWER, maxPower);
            args.putInt(OutputPowerFragment.ARGS_POWER_LEVEL, powerLevel);
            navigate(this, R.id.action_RfidSettingFragment_to_OutputPowerFragment, args);

        } else if (view == binding.onOffTimeLayout) {
            Bundle args = new Bundle();
            args.putString(OnOffTimeFragment.ARGS_ON_OFF_TIME, binding.onOffTimeTextView.getText().toString());
            args.putSerializable(OnOffTimeFragment.ARGS_PARAM, param);
            navigate(this, R.id.action_RfidSettingFragment_to_OnOffTimeFragment, args);

        } else if (view == binding.stopConditionsLayout) {
            navigate(this, R.id.action_RfidSettingFragment_to_StopConditionsFragment);

        } else if (view == binding.modulationModeLayout) {
            navigate(this, R.id.action_RfidSettingFragment_to_ModulationModeFragment);

        } else if (view == binding.encodingTypeLayout) {
            navigate(this, R.id.action_RfidSettingFragment_to_EncodingTypeFragment);

        } else if (view == binding.sessionLayout) {
            navigate(this, R.id.action_RfidSettingFragment_to_SessionFragment);

        } else if (view == binding.antiCollisionLayout) {
            navigate(this, R.id.action_RfidSettingFragment_to_AntiCollisionFragment);

        } else if (view == binding.displayTagRssiToggleButton) {
            AsReaderSDK.getInstance().getRFIDManager().setRSSIEnable(binding.displayTagRssiToggleButton.isChecked());
        }
    };

    public void onReaderInfoReceived(int onTime, int offTime, int senseTime,
                                     int lbtLevel, int fhEnable, int lbtEnable,
                                     int cwEnable, int power, int minPower, int maxPower) {
        param.readtime  = onTime;
        param.idletime  = offTime;
        param.sensetime = senseTime;
        param.lbtlevel  = lbtLevel;
        param.fhmode    = fhEnable;
        param.lbtmode   = lbtEnable;
        param.cwmode    = cwEnable;

        powerLevel  = power;
        this.minPower = minPower;
        this.maxPower = maxPower;

        if (getActivity()!=null && !isFragmentClose){
            getActivity().runOnUiThread(() -> {
                binding.outputPowerTextView.setText(String.valueOf(power / 10.0));
                binding.onOffTimeTextView.setText(onTime + "/" + offTime);
            });
        }
        if(!isFragmentClose)
            AsReaderSDK.getInstance().getRFIDManager().getAntiCollisionMode();
    }

    @Override
    public void onReceiveAntimode(int mode, int qStart, int qMax, int qMin, int counter) {
        if (getActivity()!=null && !isFragmentClose){
            getActivity().runOnUiThread(() -> {
                binding.antiCollisionTextView.setText(String.valueOf(mode));
            });
        }

        AsReaderSDK.getInstance().getRFIDManager().getChannel();
    }

    @Override
    public void didUpdateRegistry(int state) {
        if (getActivity()!=null && !isFragmentClose){
            getActivity().runOnUiThread(() -> {
                if (state == 0) {
                    showToast("Success Data Store into Registry");
                } else {
                    showToast("Fail Store into Registry");
                }
            });
        }
    }

    @Override
    public void onChannelReceived(int channel, int channelOffset) {
        if (getActivity()!=null && !isFragmentClose){
            getActivity().runOnUiThread(() -> {
                binding.rfChannelTextView.setText("ch" + channel + "off:" + channelOffset);
            });
        }
    }

    @Override
    public void onTxPowerLevelReceived(int power, int minPower, int maxPower) {
        powerLevel  = power;
        this.minPower = minPower;
        this.maxPower = maxPower;

        if (getActivity()!=null && !isFragmentClose){
            getActivity().runOnUiThread(() -> {
                float outputPower = (float)(powerLevel / 10.0);
                binding.outputPowerTextView.setText(String.valueOf(outputPower));
            });
        }
    }

    @Override
    public void onRFIDManagerError(String message) {
        Log.d(TAG, message);
    }
}
